@@ -1,3 +1,19 @@
+#   YADT - an Augmented Deployment Tool
+#   Copyright (C) 2010-2013  Immobilien Scout GmbH
+#
+#   This program is free software: you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by
+#   the Free Software Foundation, either version 3 of the License, or
+#   (at your option) any later version.
+#
+#   This program is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#   GNU General Public License for more details.
+#
+#   You should have received a copy of the GNU General Public License
+#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 from __future__ import absolute_import
 
 import twisted.internet.protocol as protocol
@@ -10,13 +26,16 @@ import logging
 import warnings
 warnings.filterwarnings("ignore", category=DeprecationWarning)
 
-
 logger = logging.getLogger('twisted')
 
-class SshFailure(failure.Failure): pass
+
+class SshFailure(failure.Failure):
+    pass
 
 
 class ProgressIndicator(object):
+    PROGRESS_LABEL = 'progress: '
+
     def __init__(self, histo_threshold=40):
         self.observables = []
         self.progress = {}
@@ -26,33 +45,26 @@ class ProgressIndicator(object):
         self.logger = logging.getLogger('progress')
 
     def update(self, observable, newvalue=None):
-#        for o in self.observables:
-#            print '%s %s %s' % (self.progress.get(o), self._render_value(self.progress.get(o)), o)
-#        print
+        observable = "justone"
         if isinstance(observable, list):
             observable = ' '.join(map(str, observable))
         if observable not in self.observables:
             self.observables.append(observable)
         if newvalue:
-            self.progress[observable] = newvalue
             self.finished.add(observable)
         else:
             value = self.progress.setdefault(observable, 0)
             if isinstance(value, int):
                 self.progress[observable] = int(value) + 1
         self._update()
-        
+
     def finish(self):
         if len(self.progress):
-            self.logger.info(self._render_compressed())
-#        for o in self.observables:
-#            print '%s %s %s' % (self.progress.get(o), self._render_value(self.progress.get(o)), o)
-#        print
-#
-#        for o in self.observables:
-#            p = self.progress[o]
-#            if isinstance(p, int):
-#                print '%s %s' % (p, o)
+            self._overwrite_remaining_progress_with_blanks()
+
+    def _overwrite_remaining_progress_with_blanks(self):
+        remaining_progress_character_count = len(self.PROGRESS_LABEL) + 1
+        print '\r' + remaining_progress_character_count * ' ' + '\r'
 
     def _render_value(self, value):
         if not value:
@@ -60,7 +72,7 @@ class ProgressIndicator(object):
         if type(value) is int:
             return self.rendered[value % len(self.rendered)]
         return value
-    
+
     def _render_compressed(self):
         rendered = [self._render_value(self.progress.get(o)) for o in self.observables]
         finished = [r for r in rendered if r not in self.rendered]
@@ -71,11 +83,12 @@ class ProgressIndicator(object):
     def _update(self):
         if sys.stderr.isatty():
             if len(self.observables) > self.histo_threshold:
-                sys.stderr.write('\rprogress: ' + self._render_compressed() + '\r')
+                sys.stderr.write('\r' + self.PROGRESS_LABEL + self._render_compressed() + '\r')
             else:
                 rendered = [self._render_value(self.progress.get(o)) for o in self.observables]
-                sys.stderr.write('\rprogress: ' + ''.join([str(o) for o in rendered]) + '\r')
-            
+                sys.stderr.write('\r' + self.PROGRESS_LABEL + ''.join([str(o) for o in rendered]) + '\r')
+
+
 class YadtProcessProtocol(protocol.ProcessProtocol):
     def __init__(self, component, cmd, pi=None, out_log_level=logging.DEBUG, err_log_level=logging.WARN, log_prefix=''):
         self.component = component
@@ -87,11 +100,11 @@ class YadtProcessProtocol(protocol.ProcessProtocol):
         self.logger = logging.getLogger(log_prefix)
         self.out_log_level = out_log_level
         self.err_log_level = err_log_level
-        
+
     def connectionMade(self):
         self.logger.debug("starting query: %s" % self.cmd)
         self.transport.write(self.cmd)
-        self.transport.closeStdin() # tell them we're done
+        self.transport.closeStdin()  # tell them we're done
         if self.pi:
             self.pi.update((self.cmd, self.component))
 
@@ -134,11 +147,12 @@ class YadtProcessProtocol(protocol.ProcessProtocol):
             reason.value.orig_protocol = self
             self.deferred.errback(reason.value)
 
+
 def stop_yadt(args=None):
     if reactor.running:
-        #reactor.callLater(1, reactor.stop)
         reactor.stop()
 
+
 def stop_and_return(return_code):
     if type(return_code) == int:
         reactor.return_code = return_code
@@ -151,10 +165,10 @@ def stop_and_return(return_code):
         reactor.return_code = 0
     stop_yadt()
 
+
 def report_error(failure, line_fun=None, include_stacktrace=True):
-    if line_fun == None:
+    if line_fun is None:
         def line_fun(line):
-            #print line
             logger.debug(line)
     if isinstance(failure, SshFailure):
         return failure
@@ -171,4 +185,3 @@ def report_error(failure, line_fun=None, include_stacktrace=True):
 def trace(arg, args='argh'):
     print 'trace: ' + args
     return arg
-
